function [obj,Y,W]= AKSC(X,m,k,c,beta,nIter)
% X: dxn input data, each column is a data
% m: reduced dimension
% k: the width of adjacency
% c: number of clusters
% nIter: number of optimization iterations

    [d,n] = size(X);
    X = X-repmat(mean(X,2),1,size(X,2)); 
    [S,~] = Construct_S_Zero(X,k);
    S = (S+ S')/2;
    S(1:n+1:end)=0;
    H = ones(n,1)*ones(1,n);
    I = diag(ones(n,1));
    
    idx = randperm(d,d)';
    W = zeros(d,m);
    W(sub2ind(size(W),idx(1:m),[1:m]')) =1;
    
    r=-100; 
    beta = 10000*beta;%rescale for easy operation
    
    B = (X'*sparse(W)*sparse(W)'*X);
    
    di = diag(B);
    ddi = repmat(di,[1,n]);
    B = B - ddi;
    B = B+B';
    
    D = B.*S + r*(H - I - S);

    Y = zeros(n,c);
    for c_idx = 1: c
        idx_s = (c_idx-1)*floor(n/c)+1;
        idx_e = floor(n/c)*c_idx;
        Y( idx_s: idx_e,c_idx) = 1; 
    end
    %shuffle Y
    rand_idx = randperm(n);
    Y = Y(rand_idx,:);
   
    for iter = 1:nIter
        %update Y
        for i = 1:n
            di = D(i,:);
            [a,idx] = max(Y'*di'); 
            tmp_Yi  = zeros(1,c); tmp_Yi(idx) = 1;
            Y(i,:)=  tmp_Yi;
        end
        N = sparse(Y)*sparse(Y');
        XN = X*sparse(N.*S +beta*S);
        XNX = XN.*X;
        diag_pnts = sum(XNX,2);
      
        [~,idx] = nth_element(-diag_pnts,m); % select nth largest elements
       
        W = zeros(d,m);
        W(sub2ind(size(W),idx(1:m),[1:m]')) =1;
        
        B = (X'*sparse(W)*sparse(W)'*X);    
   
        S = Construct_S_overD(B.*N- r*N +beta*B,k);
        S = (S+ S')/2;
        %update D
        D =B.*S + r*(H - I - S);
    end
end


function [S,D] = Construct_S_Zero(X, k)
% each column is a data

    [nFea, nSmp] = size(X);
    %D = X'*X;%
    D= L2_distance_1(X,X);
    S = zeros(nSmp,nSmp);
    %force distance to itself is the minimal
    D(1:nSmp+1:end)=-inf;
    [~,idx] = nth_element(D,k+1); % select kth largest elements
    idx = idx';
    for i = 1 : nSmp
         S(i,idx(i,2:k+1)) = 1;
    end
     D(1:nSmp+1:end)=0;
end

function [S] = Construct_S_overD(D,k)
% each column is a data
    [~, nSmp] = size(D);
    S = zeros(nSmp,nSmp);
    D(1:nSmp+1:end)=inf;
    [dumb,idx] = sort(D, 2,'descend'); % sort each row
    for i = 1 : nSmp
         S(i,idx(i,2:k+1)) = 1;
    end
end

function [d] = L2_distance_1(a,b)
    if (size(a,1) == 1)
      a = [a; zeros(1,size(a,2))]; 
      b = [b; zeros(1,size(b,2))]; 
    end

    aa=sum(a.*a);
    bb=sum(b.*b); 
    ab=a'*b; 
    d = repmat(aa',[1 size(bb,2)]) + repmat(bb,[size(aa,2) 1]) - 2*ab;

    % make sure result is all real
    d = real(d);  
end

 
 
